package cp;

import cp.tiebreak.Tiebreak;
import cp.db.DB;
import cp.db.Settings;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.Vector;

public class Player implements Comparable {
  private String lastName, firstName;
  private Club club;
  private int ratingN, ratingI, fideID, ssfID;
  private String title;
	private String sex;
	private String federation = "SWE";
	private Date birthdate;
  private int index = -1;
  private int lotNr = Integer.MAX_VALUE;
  private int withdrawnFromRound = -1;
  private int manualTiebreak = 0;
  private String group;
  private Vector games = new Vector();
  private Settings settings;
  private Tournament tournament; // Only used for sorting players

  /**
   * Constructor for available (general) player.
   */
  public Player(Settings settings) {    
    this.settings = settings;
  }
  
  /**
   * Constructor for copying an available player
   */
  public Player(Player player) throws SQLException {
    lastName = player.lastName;
    firstName = player.firstName;
    ssfID = player.ssfID;
    club = player.club;
    ratingN = player.ratingN;
    ratingI = player.ratingI;
    fideID = player.fideID;
    title = player.title;
    sex = player.sex;
    federation = player.federation;
    birthdate = player.birthdate;
    group = player.group;
    settings = player.settings;
    lastName = player.lastName;
  }

  /**
   * Constructor for tournament players. The tournament is used to sort player according to tournament settings.
   */
  public Player(Tournament tournament) {    
    this.settings = tournament.getSettings();   // Settings may be set to null if used by PH
    this.tournament = tournament;
  }
  
  public boolean hasMet(Player player) {
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (game.hasMet(player, this)) {
        return true;
      }
    }
    return false;
  }
  
  public int getWhiteCount() {
    int ans = 0;
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (this == game.getWhitePlayer()) {
        ans++;
      }
    }
    return ans;
  }
  
  public void add(Game game) {
    games.add(game);
  }
  
  public void remove(Game game) {
    games.remove(game);
  }
  
  public double getPairingScore(Round round) {
    double ans = 0;
    if (round != null) {
      for (int i=0; i<games.size(); i++) {
        Game game = (Game)games.get(i);
        ans += game.getPairingScore(this, round.getRoundNr());
      }
    }
    return ans;
  }
  
  public double getScore(Round round) {
    double ans = 0;
    if (round != null) {
      for (int i=0; i<games.size(); i++) {
        Game game = (Game)games.get(i);
        ans += game.getScore(this, round.getRoundNr());
      }
    }
    return ans;
  }

  public int getLotNr() {
    return lotNr;
  }
  
  public String toString() {
    String ans = "";
    if (title != null && !title.trim().isEmpty()) {
      ans = title + " "; 
    }
    if (settings != null) {
      Settings.PlayerPresentation pres = settings.getPlayerPresentation(); 
      if (pres == cp.db.Settings.PlayerPresentation.FIRST_LAST) {
        ans = String.format("%s%s %s", ans, firstName.trim(), lastName.trim()); 
      }
      else if (pres == cp.db.Settings.PlayerPresentation.LAST_FIRST) {
        ans = String.format("%s%s %s", ans, lastName.trim(), firstName.trim()); 
      }
    }
    else {
      ans = String.format("%s%s %s", ans, firstName.trim(), lastName.trim()); 
    }
    if (withdrawnFromRound > -1) {
      ans = String.format("%s (utgtt r%d)", ans, withdrawnFromRound); 
    } 
    return ans; 
  }

  public int getIndex() {
    return index;
  }
  
  /*
   * Used by SSF tournament system
   */
  public void setIndex(int index) {
    this.index = index;
  }  
  
  public void load(ResultSet rs) throws SQLException {
		index = rs.getInt("index");
    lastName = rs.getString("lastname");
    firstName = rs.getString("firstname");
    ssfID = rs.getInt("ssfid");
		club = settings.getClubs().getClub(rs.getInt("clubindex"));
    ratingN = rs.getInt("ratingn");
		ratingI = rs.getInt("ratingi");
    title = rs.getString("title");
		sex = rs.getString("sex");
		federation = rs.getString("federation");
		fideID = rs.getInt("fideid");
		birthdate = rs.getDate("birthdate");
    group = rs.getString("playergroup");
  }

  public void loadFromTournament(ResultSet rs) throws SQLException {
    index = rs.getInt("index");
    withdrawnFromRound = rs.getInt("withdrawnfromround");
    manualTiebreak = rs.getInt("manualtiebreak");
  }

  public void add(Tournament tournament) throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "INSERT INTO tournamentplayers " + "(lastname, firstname, ssfID, clubindex, ratingn, ratingi, title, playergroup, sex, federation, fideid, birthdate, tournamentindex, withdrawnfromround, manualtiebreak) " + "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
    PreparedStatement s = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
    s.setString(1, lastName);
    s.setString(2, firstName);
    s.setInt(3, ssfID);
    if (club != null) {
      s.setInt(4, club.getIndex());
    }
    else {
      s.setNull(4, Types.INTEGER);
    }
    s.setInt(5, ratingN);
		s.setInt(6, ratingI);
    s.setString(7, title);
    s.setString(8, group);
		s.setString(9, sex);
		s.setString(10, federation);
		s.setInt(11, fideID);
		s.setDate(12, birthdate);
    s.setInt(13, tournament.getIndex());
    s.setInt(14, withdrawnFromRound);
    s.setInt(15, manualTiebreak);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
      // Read the generated index
      ResultSet rs = s.getGeneratedKeys();
      rs.next();
      index = rs.getInt(1);
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }
  
  public void addToAvailablePlayers() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "INSERT INTO availableplayers " + "(lastname, firstname, ssfid, clubindex, ratingn, ratingi, title, sex, federation, fideid, birthdate, playergroup) " + "VALUES (?,?,?,?,?,?,?,?,?,?,?,?)";
    PreparedStatement s = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
    s.setString(1, lastName);
    s.setString(2, firstName);
    s.setInt(3, ssfID);
    if (club != null) {
      s.setInt(4, club.getIndex());
    }
    else {
      s.setNull(4, Types.INTEGER);
    }
    s.setInt(5, ratingN);
		s.setInt(6, ratingI);
    s.setString(7, title);
		s.setString(8, sex);
		s.setString(9, federation);
		s.setInt(10, fideID);
		s.setDate(11, birthdate);
    s.setString(12, group);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
      // Read the generated index
      ResultSet rs = s.getGeneratedKeys();
      rs.next();
      index = rs.getInt(1);
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void update(Tournament tournament) throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "UPDATE tournamentplayers " + "SET lastname=?, firstname=?, ssfid=?, clubindex=?, ratingn=?, ratingi=?, title=?, playergroup=?, sex=?, federation=?, fideid=?, birthdate=?, withdrawnfromround=?, manualtiebreak=?" + "WHERE tournamentindex=? AND index=?";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setString(1, lastName);
    s.setString(2, firstName);
    s.setInt(3, ssfID);
    if (club != null) {
      s.setInt(4, club.getIndex());
    }
    else {
      s.setNull(4, Types.INTEGER);
    }
    s.setInt(5, ratingN);
		s.setInt(6, ratingI);
    s.setString(7, title);
    s.setString(8, group);
		s.setString(9, sex);
		s.setString(10, federation);
		s.setInt(11, fideID);
		s.setDate(12, birthdate);
    s.setInt(13, withdrawnFromRound);
    s.setInt(14, manualTiebreak);
    s.setInt(15, tournament.getIndex());
    s.setInt(16, index);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void update() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "UPDATE availableplayers " + "SET lastname=?, firstname=?, ssfid=?, clubindex=?, ratingn=?, ratingi=?, title=?, playergroup=?, sex=?, federation=?, fideid=?, birthdate=? " + "WHERE index=?";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setString(1, lastName);
    s.setString(2, firstName);
    s.setInt(3, ssfID);
    if (club != null) {
      s.setInt(4, club.getIndex());
    }
    else {
      s.setNull(4, Types.INTEGER);
    }
    s.setInt(5, ratingN);
		s.setInt(6, ratingI);
    s.setString(7, title);
    s.setString(8, group);
		s.setString(9, sex);
		s.setString(10, federation);
		s.setInt(11, fideID);
		s.setDate(12, birthdate);
    s.setInt(13, index);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void setLotNr(int lotNr) {
    this.lotNr = lotNr;
  }

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public void setClub(Club club) {
    this.club = club;
  }

  public void setRatingN(int rating) {
    this.ratingN = rating;
  }

	public void setRatingI(int rating) {
		this.ratingI = rating;
	}

  public String getLastName() {
    return lastName;
  }

  public String getFirstName() {
    return firstName;
  }
  
  public Club getClub() {
    return club;
  }

  public int getRatingN() {
    return ratingN;
  }
  
	public int getRatingI() {
		return ratingI;
	}
  
  public void setGroup(String group) {
    this.group = group;
  }
  
  public String getGroup() {
    return group;
  }

  public double getRatingNChange() {
    double ratingChange = 0;
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      ratingChange += RatingSSF.calculateRatingChange(this, game);
    }
    return ratingChange;
  }

  public void remove() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "DELETE FROM availableplayers WHERE index=?";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setInt(1, index);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void remove(Tournament tournament) throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "DELETE FROM tournamentplayers WHERE index=? AND tournamentindex=?";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setInt(1, index);
    s.setInt(2, tournament.getIndex());
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void setWithdrawnFromRound(int withdrawnFromRound) {
    this.withdrawnFromRound = withdrawnFromRound;
  }

  public int getWithdrawnFromRound() {
    return withdrawnFromRound;
  }

  /**
   * Compares players according to rating according to ratingN or tournament settings (if no rating exist, the name).
   */
  public int compareTo(Object o) {
    int ratingThis, ratingB;
    Player b = (Player) o;
    if (tournament != null) {
      // Use tournament settings rating
      ratingThis = tournament.getRating(this);
      ratingB = tournament.getRating(b);
    }
    else {
      // Use national rating
      ratingThis = ratingN;
      ratingB = b.getRatingN();
    }
    if (ratingThis > ratingB) {
      return -1;
    }
    else if (ratingB > ratingThis) {
      return 1;
    }
    else {
      if (ratingThis > 0 && ratingB > 0) {
        return 0;  
      }
      else {
        return toString().compareTo(b.toString());
      }
    }
  }
  
  public double getTiebreak(Tiebreak tiebreak, Round round) {
    return tiebreak.getTiebreak(round, this);
  }

  public boolean isWithdrawn(int round) {
    return (withdrawnFromRound != -1 && round >= withdrawnFromRound);
  }

  public void setTitle(String title) {
    this.title = title;
  }

  public String getTitle() {
    return title;
  }

  public Vector getGames() {
    return games;
  }
  
  public int getManualTiebreak() {
    return manualTiebreak;
  }

  public void setManualTiebreak(int manualTiebreak) {
    this.manualTiebreak = manualTiebreak;
  }

	public String getSex() {
		return sex;
	}

	public String getFederation() {
		return federation;
	}

	public int getFideID() {
		return fideID;
	}

  public int getSSFID() {
    return ssfID;
  }

	public Date getBirthdate() {
		return birthdate;
	}

	public void setSex(String sex) {
		this.sex = sex;
	}

	public void setFederation(String federation) {
		this.federation = federation;
	}

	public void setFIDENr(int fideNr) {
		this.fideID = fideNr;
	}

  public void setSSFID(int ssfID) {
    this.ssfID = ssfID;
  }

	public void setBirthdate(Date birthdate) {
		this.birthdate = birthdate;
	}
  
  public void setTournament(Tournament tournament) {
    this.tournament = tournament;
  }
  
  boolean hasELO() {
    return ratingI > 0;
  }

  /*
   * Export player to text file
   */
  void export(FileWriter writer) throws IOException {
    String str = String.format("%s\t%s\t%s\r\n", lastName != null ? lastName : "", firstName != null ? firstName : "", club != null ? club.toString() : "");
    writer.write(str);
  }

  /*
   * Import player from text file
   */
  void importFromFile(String str) throws SQLException {
    try {
      Scanner sc = new Scanner(str).useDelimiter("\t");
      setLastName(sc.next().replace("\u00A0"," ").trim());
      setFirstName(sc.next().replace("\u00A0"," ").trim());
      String clubName = sc.next().replace("\u00A0"," ").trim();
      // Create club in DB if it doesnt exist
      Clubs clubs = settings.getClubs();
      Club club = clubs.getClub(clubName);
      if (club == null) {
        clubs.addToDB(clubName);
        club = clubs.getClub(clubName);
      }
      setClub(club);
    }
    catch (NoSuchElementException e) {
      throw new NoSuchElementException("Rad '" + str + "' r inte korrekt. ");
    }
    catch (SQLException e) {
      throw new SQLException("Rad '" + str + "' r inte korrekt. " + e.getMessage());
    }
  }

  public void clear() {
    lastName = null;
    firstName = null;
    club = null;
    ratingN = 0;
    ratingI = 0;
    fideID = 0;
    ssfID = 0;
    title = null;
    sex = null;
    federation = "SWE";
    birthdate = null;
  }
}
