package cp;

public class RatingSSF {
  
  public static double calculateRatingChange(Player player, Game game) {
    if (game.getResult().getResultType() == Result.ResultType.WHITE_WIN || 
    game.getResult().getResultType() == Result.ResultType.BLACK_WIN || 
    game.getResult().getResultType() == Result.ResultType.DRAW) {
      if (game.getWhitePlayer() == player && game.getBlackPlayer() != null) {
        return calculateWhiteChange(game.getWhitePlayer().getRatingN(), game.getBlackPlayer().getRatingN(), game.getResult());
      }
      else if (game.getBlackPlayer() == player && game.getWhitePlayer() != null) {
        return calculateBlackChange(game.getWhitePlayer().getRatingN(), game.getBlackPlayer().getRatingN(), game.getResult());
      }
    }
    return 0;
  }

  private static double calculateWhiteChange(int ratingWhite, int ratingBlack, Result result) {
    double change = getTableValue(Math.abs(ratingWhite - ratingBlack));
    if (result.getResultType() == Result.ResultType.WHITE_WIN) {
      if (ratingWhite < ratingBlack) {
        change = 32 - change;
      }
    }
    else if (result.getResultType() == Result.ResultType.BLACK_WIN) {
      if (ratingWhite > ratingBlack) {
        change = -(32 - change);
      }
      else {
        change = -change;
      }
    }
    else if (result.getResultType() == Result.ResultType.DRAW) {
      change = change - 16;
      if (ratingWhite < ratingBlack) {
        change = -change;
      }
    }
    
    if (ratingWhite > 2200) {
      change = change/2;
    }
    return change;    
  }
  
  private static double calculateBlackChange(int ratingWhite, int ratingBlack, Result result) {
    double change = getTableValue(Math.abs(ratingWhite - ratingBlack));
    if (result.getResultType() == Result.ResultType.BLACK_WIN) {
      if (ratingWhite > ratingBlack) {
        change = 32 - change;
      }
    }
    else if (result.getResultType() == Result.ResultType.WHITE_WIN) {
      if (ratingWhite < ratingBlack) {
        change = -(32 - change);
      }
      else {
        change = -change;
      }
    }
    else if (result.getResultType() == Result.ResultType.DRAW) {
      change = change - 16;
      if (ratingWhite > ratingBlack) {
        change = -change;
      }
    }

    if (ratingBlack > 2200) {
      change = change/2;
    }
    return change;    
  }

  private static int getTableValue(int diff) {
    int change;
    if (diff <=10) {
      change = 16;
    }
    else if (diff <= 33) {
      change = 15;
    }
    else if (diff <= 56) {
      change = 14;
    }
    else if (diff <= 79) {
      change = 13;
    }
    else if (diff <= 102) {
      change = 12;
    }
    else if (diff <= 126) {
      change = 11;
    }
    else if (diff <= 151) {
      change = 10;
    }
    else if (diff <= 178) {
      change = 9;
    }
    else if (diff <= 207) {
      change = 8;
    }
    else if (diff <= 236) {
      change = 7;
    }
    else if (diff <= 270) {
      change = 6;
    }
    else if (diff <= 308) {
      change = 5;
    }
    else if (diff <= 352) {
      change = 4;
    }
    else if (diff <= 409) {
      change = 3;
    }
    else if (diff <= 499) {
      change = 2;
    }
    else {
      change = 1;
    }
    return change;
  }
}
