package cp.db;

import java.sql.*;
import javax.swing.JOptionPane;

public class DB {
  private static Connection conn;
  private String driver = "org.apache.derby.jdbc.EmbeddedDriver";
  private int dbVersion = 12;

  public DB() throws SQLException, ClassNotFoundException {
    try {
      connect("database");
    }
    catch (SQLException e) {
      if (e.getMessage().indexOf("not found") > 0) {
        // Create a new database
        JOptionPane.showMessageDialog(null,"Lotta behver en databas fr att fungera. Lotta kommer nu att skapa en mapp som heter 'database' dr programmet ligger.", "Skapa databas", JOptionPane.INFORMATION_MESSAGE);
        create("database");
      }
      else {
        throw e;
      }
    }
    // Update database if necessary
    updateTo(dbVersion);
  }

  public static Connection getConnection() {
    return conn;
  }
  
  /**
   *Connects to a database. 
   */
  private void connect(String dbName) throws SQLException, ClassNotFoundException {
    String connectionURL = "jdbc:derby:" + dbName;
    conn = DriverManager.getConnection(connectionURL);
    conn.setAutoCommit(false);
  }

  /**
   *Creates a new database. 
   */
  private void create(String dbName) throws SQLException {
    String connectionURL = "jdbc:derby:" + dbName + ";create=true";
    try {
      conn = DriverManager.getConnection(connectionURL);
      conn.setAutoCommit(false);
      // Create all tables
      createClubTable();
      createAvailablePlayersTable();
      createTournamentsTable();
      createTournamentPlayersTable();
      createTournamentGamesTable();
      createTournamentRoundDatesTable();
      createSettingsTable();
      createStringSettingsTable();
      createTournamentTiebreaksTable();
			Settings.setDBVersion(dbVersion);
      Settings.setUpdateLocation("http://privat.bahnhof.se/wb143434/Lotta/lotta.xml");
      conn.commit();
    }
    catch (SQLException e) {
      JOptionPane.showMessageDialog(null, "Fel:" + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
      conn.rollback();
    }
  }

  /**
   * Closes the database
   */
  public void close() throws SQLException {
    if (conn != null) {
      conn.commit();
      conn.close();
    }
  }

  /**
   * Closes the embedded database.
   */
  public void closeDatabase() throws SQLException {
    /*** In embedded mode, an application should shut down Derby.
                Shutdown throws the XJ015 exception to confirm success. ***/
    if (driver.equals("org.apache.derby.jdbc.EmbeddedDriver")) {
      try {
        DriverManager.getConnection("jdbc:derby:;shutdown=true");
      }
      catch (SQLException se) {
        if (!se.getSQLState().equals("XJ015")) {
          JOptionPane.showMessageDialog(null, "Databasen stngdes inte ner korrekt", "Databasfel", JOptionPane.ERROR_MESSAGE);
        }
      }
			catch (Exception se) {
				JOptionPane.showMessageDialog(null, "Databasen stngdes inte ner korrekt: " + se.getMessage(), "Databasfel", JOptionPane.ERROR_MESSAGE);
			}
    }
  }

  /**
   * Creates the table that contains available players
   */
  private void createAvailablePlayersTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE availableplayers (" +
          "index int GENERATED ALWAYS AS IDENTITY, " +
          "lastname varchar(255) not null, " +
          "firstname varchar(255), " +
          "ssfid int, " +
          "clubindex int, " +
          "ratingn int, " +
          "ratingi int, " +
          "title varchar(16), " +
          "sex varchar(1), " +
          "federation varchar(3), " +
          "fideid int, " +
          "birthdate date, " +
          "playergroup varchar(255), " +
          "PRIMARY KEY (index), " +
          "FOREIGN KEY (clubindex) REFERENCES clubs(index) " + 
          ")";
    s.execute(sql);
    sql = "CREATE UNIQUE INDEX availableplayers_idx " + 
          "ON availableplayers (lastname, firstname, clubindex)";
    s.execute(sql);
    s.close();
  }

  /**
   * Creates the table that contains the tournaments
   */
  private void createTournamentsTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE tournaments (" + 
          "tournament varchar(255) not null, " + 
          "tournamentgroup varchar(255) not null, " + 
          "index int GENERATED ALWAYS AS IDENTITY, " + 
          "pairingsystem varchar(32) not null, " + 
          "initialpairing varchar(32) not null, " + 
          "rounds int not null, " + 
          "barredpairing varchar(5) not null, " +
          "compensateweakplayerpp varchar(5) not null, " + 
          "chess4 varchar(5) not null, " + 
          "pointspergame int not null, " + 
          "resultspage varchar(255), " + 
          "standingspage varchar(255), " +
          "playerlistpage varchar(255), " + 
          "roundforroundpage varchar(255), " + 
          "clubstandingspage varchar(255), " +
          "ratingchoice varchar(32) not null, " + 
          "showlask varchar(5), " + 
          "showelo varchar(5), " + 
					"showgroup varchar(5), " + 
          "city varchar(256), " + 
          "startdate date, " + 
          "enddate date, " + 
          "chiefarbiter varchar(256), " + 
          "deputyarbiter varchar(256), " + 
          "timecontrol varchar(256), " + 
          "federation varchar(3), " + 
          "PRIMARY KEY (index) " + ")";
    s.execute(sql);
    sql = "CREATE UNIQUE INDEX tournaments_idx " + 
          "ON tournaments (tournament, tournamentgroup)";
    s.execute(sql);
    s.close();
  }

  /**
   * Deletes a tournament.
   */

  /**
   * Creates the table that contains the players in the tournaments
   */
  private void createTournamentPlayersTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE tournamentplayers (" +
          "index int GENERATED ALWAYS AS IDENTITY, " +
          "lastname varchar(255) not null, " +
          "firstname varchar(255), " +
          "ssfid int, " +
          "clubindex int, " +
          "ratingn int, " +
          "ratingi int, " +
          "title varchar(3), " +
          "playergroup varchar(255), " +
          "sex varchar(1), " +
          "federation varchar(3), " +
          "fideid int, " +
          "birthdate date, " +
          "tournamentindex int not null, " +
          "withdrawnfromround int, " +
          "manualtiebreak int, " +
          "PRIMARY KEY (index), " +
          "FOREIGN KEY (tournamentindex) REFERENCES tournaments(index), " + 
          "FOREIGN KEY (clubindex) REFERENCES clubs(index) " + 
          ")";
    s.execute(sql);
    sql = "CREATE UNIQUE INDEX tournamentplayers_idx " + 
          "ON tournamentplayers (lastname, firstname, clubindex, tournamentindex)";
    s.execute(sql);
    s.close();
  }

  /**
   * Creates the table that contains the tiebrakingsystems in the tournaments
   */
  private void createTournamentTiebreaksTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE tournamenttiebreaks (" +
          "index int GENERATED ALWAYS AS IDENTITY, " +    // Used only to preserve order.
          "tiebreak varchar(255) not null, " +
          "tournamentindex int not null, " +
          "PRIMARY KEY (tiebreak, tournamentindex), " +
          "FOREIGN KEY (tournamentindex) REFERENCES tournaments(index) " + 
          ")";
    s.execute(sql);
    s.close();
  }

  /**
   * Creates the table that contains the games in the tournaments
   */
  private void createTournamentGamesTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE tournamentgames (" +
          "tournament int not null, " +
          "round int not null, " +
          "boardnr int not null, " +
          "whiteplayer int, " +
          "blackplayer int, " +
          "resulttype int not null, " +
          "whitescore double not null, " +
          "blackscore double not null, " +
          "whiteplayerlotnr int, " +
          "blackplayerlotnr int, " +
          "PRIMARY KEY (tournament, round, boardnr), " +
          "FOREIGN KEY (tournament) REFERENCES tournaments(index), " + 
          "FOREIGN KEY (whiteplayer) REFERENCES tournamentplayers(index), " + 
          "FOREIGN KEY (blackplayer) REFERENCES tournamentplayers(index) " + 
          ")";
    s.execute(sql);
    s.close();
  }
  
  /**
   * Creates the table that contains information about the when the rounds are played in the tournament.
   */
  private void createTournamentRoundDatesTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE tournamentrounddates (" + 
                 "tournament int not null, " + 
                 "round int not null, " + 
                 "rounddate date, " + 
                 "PRIMARY KEY (tournament, round), " + 
                 "FOREIGN KEY (tournament) REFERENCES tournaments(index)" + ")";
    s.execute(sql);
    s.close();
  }
  private void createClubTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE clubs (" +
          "index int GENERATED ALWAYS AS IDENTITY, " +  
          "club varchar(255) not null, " +
          "chess4members int, " +
          "PRIMARY KEY (index) " +
          ")";
    s.execute(sql);
    sql = "CREATE UNIQUE INDEX clubs_idx " + 
          "ON clubs (club)";
    s.execute(sql);
    s.close();
  }
  
  /**
   * Creates a settings table that contains meta info.
   */
  private void createSettingsTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE settings (" +
          "setting varchar(255) not null, " + 
          "value int, " +
          "PRIMARY KEY (setting) " +
          ")";
    try {
      s.execute(sql);
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }
  
  /**
   * Creates a settings table that contains meta info strings.
   */
  private void createStringSettingsTable() throws SQLException {
    Statement s = conn.createStatement();
    String sql = "CREATE TABLE stringsettings (" +
          "setting varchar(255) not null, " + 
          "value varchar(255), " +
          "PRIMARY KEY (setting) " +
          ")";
    try {
      s.execute(sql);
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }

  /**
   * Updates the current database to a specified version
   */
  private void updateTo(int version) throws SQLException {
    if (version > 1) {
      int oldVersion = Settings.getDBVersion();
      if (oldVersion == 1) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournaments " +
              "ADD compensateweakplayerpp varchar(5) not null default 'false'";
        s.execute(sql);
        s.close();
				Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 2) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE availableplayers " +
              "ADD playergroup varchar(255)";
        s.execute(sql);
        sql = "ALTER TABLE tournamentplayers " +
              "ADD playergroup varchar(255)";
        s.execute(sql);
        s.close();
				Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 3) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournaments " +
              "ADD chess4 varchar(5)  not null default 'false' ";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " +
              "ADD clubstandingspage varchar(255)";
        s.execute(sql);
        sql = "ALTER TABLE clubs " +
              "ADD chess4members int";
        s.execute(sql);
        s.close();
				Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 4) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournaments " +
              "ADD chess4 varchar(5)  not null default 'false' ";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " +
              "ADD clubstandingspage varchar(255)";
        s.execute(sql);
        sql = "ALTER TABLE clubs " +
              "ADD chess4members int";
        s.execute(sql);
        s.close();
				Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 4 && version == 5) {
        JOptionPane.showMessageDialog(null, "Databasuppgradering ej mjlig. Vnligen skapa en ny databas fr denna version av ChessPairer.", "Databasfel", JOptionPane.ERROR_MESSAGE);
        System.exit(0);
      }
      if (oldVersion == 5) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournamentplayers " +
              "ADD sex varchar(1)";
        s.execute(sql);
        sql = "ALTER TABLE tournamentplayers " +
              "ADD federation varchar(3)";
        s.execute(sql);
        sql = "ALTER TABLE tournamentplayers " +
              "ADD fideid int";
        s.execute(sql);
        sql = "ALTER TABLE tournamentplayers " +
              "ADD birthdate date";
        s.execute(sql);
        sql = "ALTER TABLE availableplayers " +
              "ADD sex varchar(1)";
        s.execute(sql);
        sql = "ALTER TABLE availableplayers " +
              "ADD federation varchar(3)";
        s.execute(sql);
        sql = "ALTER TABLE availableplayers " +
              "ADD fideid int";
        s.execute(sql);
        sql = "ALTER TABLE availableplayers " +
              "ADD birthdate date";
        s.execute(sql);
        s.close();
				Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 6) {
        createStringSettingsTable();
        Settings.setDBVersion(oldVersion+1);
        conn.commit();
        updateTo(version);
      }
      if (oldVersion == 7) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournaments " + "ADD ratingchoice varchar(32) not null DEFAULT 'LASK'";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD showlask varchar(5) not null default 'true' ";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD showelo varchar(5) not null default 'false' ";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD showgroup varchar(5) not null default 'true' ";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD city varchar(256)";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD startdate date";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD enddate date";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD chiefarbiter varchar(256)";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD deputyarbiter varchar(256)";
        s.execute(sql);
        sql = "ALTER TABLE tournaments " + "ADD timecontrol varchar(256)";
        s.execute(sql);
        createTournamentRoundDatesTable();
        Settings.setDBVersion(oldVersion + 1);
        conn.commit();
        updateTo(version);
      }      
      if (oldVersion == 8) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE tournaments " + "ADD federation varchar(3)";
        s.execute(sql);
        Settings.setDBVersion(oldVersion + 1);
        conn.commit();
        updateTo(version);
      }      
      if (oldVersion == 9) {
        // Version 9 was set by mistake.
        Settings.setDBVersion(oldVersion + 1);
        conn.commit();
        updateTo(version);
      }      
      if (oldVersion == 10) {
        Statement s = conn.createStatement();
        String sql = "ALTER TABLE availableplayers " + "ADD ssfid int";
        s.execute(sql);
        sql = "ALTER TABLE tournamentplayers " + "ADD ssfid int";
        s.execute(sql);
        Settings.setDBVersion(oldVersion + 1);
        conn.commit();
        updateTo(version);
      } 
      if (oldVersion == 11) {
        Settings.setUpdateLocation("http://privat.bahnhof.se/wb143434/Lotta/lotta.xml");
        Settings.setDBVersion(oldVersion+1);
        updateTo(version);
      }      
      
      // TODO: Kommande ndring
      /*        Statement s = conn.createStatement();
              String sql = "ALTER TABLE tournaments " +
                    "ALTER COLUMN tournamentgroup null";
              s.execute(sql);*/
    }
  }
  
}

