package cp.gui;

import cp.*;
import java.awt.event.*;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.text.MessageFormat;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class AlphabeticalPairingPanel extends JScrollPane {
  private AlphabeticalPairingTableModel tableModel = new AlphabeticalPairingTableModel();
  private JTable jTable = new JTable(tableModel);
  private JTable jPrintTable = new JTable(tableModel);
  private TableRowSorter<AlphabeticalPairingTableModel> sorter;
  private String[] colors = new String[] {"white", "aqua", "beige", "pink", "moccasin", "lime", "lightgray", "magenta", "lightblue", "wheat", "orange", "silver", "red", "lightcyan", "yellow", "springgreen"};

  public AlphabeticalPairingPanel() {
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setViewportView(jTable);
    jTable.setFillsViewportHeight(true);
    setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    this.addComponentListener(new ComponentAdapter() {
      public void componentShown(ComponentEvent e) {
        this_componentShown();
      }
    });
    jTable.setShowGrid(false);
    jTable.setShowHorizontalLines(true);
    jPrintTable.setShowGrid(false);
    jPrintTable.setShowHorizontalLines(true);
    // Setup column sorting
    sorter = new TableRowSorter<AlphabeticalPairingTableModel>(tableModel);
    jTable.setRowSorter(sorter);
    jPrintTable.setRowSorter(sorter);
    List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
    sortKeys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING)); // Klass
    sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING)); // Namn
    sorter.setSortKeys(sortKeys);
  }

  public void setRound(Round round) {
    tableModel.setRound(round);
    setColumnWidths();
  }

  public void print() throws PrinterException {
    update();
    Round round = tableModel.getRound();
    if (round != null) {
      if (round.getTournament().isChess4()) {
        if (JOptionPane.showConfirmDialog(this, "Vill du ha varje klass p en egen sida?", "Utskriftsval", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.OK_OPTION) {
          printClasses(round);
        }
        else {
          MessageFormat headerFormat = new MessageFormat(String.format("Alfabetisk lottning fr rond %d", round.getRoundNr()));
          MessageFormat footerFormat = new MessageFormat(String.format("%s, grupp %s\n     - Sida {0} -", round.getTournament(), round.getTournament().getGroup()));
          EditResultsPanel.optimizeTableWidth(jPrintTable);
          jPrintTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, footerFormat);
        }
      }
    }
  }
  
/*
 * Prints each chess4 class on separate pages
 */
  private void printClasses(Round round) throws PrinterException {
    PrinterJob printerJob = PrinterJob.getPrinterJob();
    if (printerJob.printDialog()) {
      try {
        Vector<Club> clubs = round.getTournament().getCurrentClubList();
        Collections.sort(clubs);
        for (int i=0; i<clubs.size(); i++) {
          Club club = clubs.get(i);
          MessageFormat headerFormat = new MessageFormat(String.format("Alfabetisk lottning fr rond %d", round.getRoundNr()));
          MessageFormat footerFormat = new MessageFormat(String.format("%s, grupp %s, klass %s\n     - Sida {0} -", round.getTournament(), round.getTournament().getGroup(), club));
          tableModel.setClub(club);
          EditResultsPanel.optimizeTableWidth(jPrintTable);
          jPrintTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, footerFormat, false, null, false, printerJob.getPrintService());
        }
      }
      catch (PrinterException e) {
        throw e;
      }
      finally {
        tableModel.setClub(null);
      }
    }
  }

  public Round getRound() {
    return tableModel.getRound();
  }

  public void update() {
    if (tableModel != null) {
      tableModel.update();
      setColumnWidths();
      jTable.doLayout();
      jTable.repaint();
    }
  }

  private void setColumnWidths() {
    setColumnWidths(jTable);
    setColumnWidths(jPrintTable);
  }

  private void setColumnWidths(JTable table) {
    TableColumn column = null;
    for (int i = 0; i < table.getColumnModel().getColumnCount(); i++) {
      column = table.getColumnModel().getColumn(i);
      if (i == 0) {
        column.setPreferredWidth(200);
      }
      else if (i == 1) {
        column.setPreferredWidth(150);
      }
      else if (i == 2) {
        column.setPreferredWidth(40);
      }
      else if (i == 3) {
        column.setPreferredWidth(40);
      }
      else if (i == 4) {
        column.setPreferredWidth(40);
      }
    }
  }

  private void insertHeaders(Document document, Element table, int columns) {
    // Create headers
    Element tr = document.createElement("tr");
    table.appendChild(tr);
    tr.setAttribute("class", "CP_TableAlphabeticalHeader");
    for (int cols = 0; cols < columns; cols++) {
      for (int i = 0; i < jTable.getColumnCount(); i++) {
        Element th = document.createElement("th");
        tr.appendChild(th);
        th.setAttribute("class", String.format("CP_TableAlphabeticalHeaderC%d", i + 1));
        Text text = document.createTextNode(jTable.getColumnName(i));
        th.appendChild(text);
      }
    }
  }

  public void save(Document document, Element body, int maxRows, int columns) throws ChessPairerException {
    boolean isChess4 = tableModel.getRound().getTournament().isChess4();
    if (tableModel.getRound() != null && isChess4 && !jTable.getColumnName(sorter.getSortKeys().get(0).getColumn()).equals("Klubb")) {
      JOptionPane.showMessageDialog(this, "Lottningen publiceras sorterad p " + jTable.getColumnName(sorter.getSortKeys().get(0).getColumn()) + ".\nSortering p klubb r att fredra fr schackfyran!\nKlicka i s fall p Klubb-kolumnen i Alfabetisk lottning.", "Information", JOptionPane.INFORMATION_MESSAGE);
    }
    else {
      JOptionPane.showMessageDialog(this, "Lottningen publiceras sorterad p " + jTable.getColumnName(sorter.getSortKeys().get(0).getColumn()), "Information", JOptionPane.INFORMATION_MESSAGE);
    }
    // Insert table
    Element table = document.createElement("table");
    body.appendChild(table);
    table.setAttribute("class", "CP_TableAlphabetical");
    table.setAttribute("cellpadding", "0");
    table.setAttribute("cellspacing", "0");
    table.setAttribute("border", "0");

    int modelOffset = 0;
    // Calculate nr of rows needed
    int neededRows = (int) Math.ceil(jTable.getRowCount() / (double) columns);
    int rows = neededRows;
    if (neededRows > maxRows) {
      rows = Math.min(neededRows, maxRows);
    }
    // Create rows
    Vector<Club> clubs = tableModel.getRound().getTournament().getCurrentClubList();
    Collections.sort(clubs);    
    for (int repeat = 0; repeat < Math.ceil(jTable.getRowCount() / ((double) rows * columns)); repeat++) {
      insertHeaders(document, table, columns);
      for (int i = 0; i < rows; i++) {
        createPlayersRow(document, table, columns, rows, i, modelOffset, isChess4, clubs);
      }
      modelOffset += columns * rows;
    }
  }

  private void createPlayersRow(Document document, Element table, int columns, int rows, int row, int modelOffset, boolean isChess4, Vector<Club> clubs) {
    Element tr = document.createElement("tr");
    table.appendChild(tr);
    tr.setAttribute("class", "CP_TableAlphabeticalRow");
    // Create columns
    for (int column = 0; column < columns; column++) {
      // Create subcolumns in each row
      createPlayersColumn(document, tr, column, row, column * rows + modelOffset, isChess4, clubs);
    }
  }

  private void createPlayersColumn(Document document, Element tr, int column, int row, int modelOffset, boolean isChess4, Vector<Club> clubs) {
    for (int j = 0; j < jTable.getColumnCount(); j++) {
      Element th = document.createElement("th");
      int index = row + modelOffset;
      if (jTable.getRowCount() > index) {
        if (isChess4) {
          // Get an appropriate color
          Club club = (Club) jTable.getValueAt(index, 1);
          int idx = clubs.indexOf(club);
          int colorIndex = idx % colors.length;
          String color = colors[colorIndex];
          th.setAttribute("class", String.format("CP_TableAlphabeticalRowC%d%s", j + 1, color));
        }
        else {
          // Use gray/white background colors
          boolean even = column % 2 == 1;
          th.setAttribute("class", String.format("CP_TableAlphabeticalRowC%d%s", j + 1, even ? "lightgray" : "white"));
        }
        Object obj = jTable.getValueAt(index, j);
        if (obj != null) {
          Text text = document.createTextNode(obj.toString());
          th.appendChild(text);
        }
        tr.appendChild(th);
      }
    }
  }

  /**
   * When the table is shown, set the round again so the player information is correct.
   */
  private void this_componentShown() {
    update();
  }
}
