package cp.gui;

import cp.Club;
import cp.Game;
import cp.Games;
import cp.Player;
import cp.Players;
import cp.Round;
import javax.swing.table.*;

public class AlphabeticalPairingTableModel extends AbstractTableModel {
  private String[] columnNames = { "Namn", "Klubb", "Bord" };
  private Round round;
  private Players players;
  private Club club;

  void setRound(Round round) {
    this.round = round;
    update();
  }

  /**
   * Updates the alphabetical pairing so that any changes will be visible (when adding or removing boards in the current round).
   * Update also filter players (when printing each class individually).
   */
  void update() {
    if (round != null) {
      players = (Players) round.getPairedPlayers().clone();
    }
    else {
      players = null;
    }
    filter(players);
    fireTableDataChanged();
  }

  public String getColumnName(int col) {
    return columnNames[col].toString();
  }

  public int getRowCount() {
    if (players != null) {
      return players.size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (round != null) {
      Player player = players.get(rowIndex);
      if (columnIndex == 0) {
        return player.toString();
      }
      else if (columnIndex == 1) {
        return player.getClub();
      }
      else if (columnIndex == 2) {
        Games games = round.getGames();
        if (games != null) {
          Game game = games.getGame(player);
          if (game != null) {
            if (game.plays(null)) {
              return "Fri";
            }
            if (game.getWhitePlayer() == player) {
              return String.format("%d V", game.getBoardNr());
            }
            else {
              return String.format("%d S", game.getBoardNr());
            }
          }
        }
      }
      else if (columnIndex == 3) {
        Games games = round.getGames();
        if (games != null) {
          Game game = games.getGame(player);
          if (game != null) {
            if (game.getWhitePlayer() == player) {
              return "Vit";
            }
            else {
              return "Svart";
            }
          }
        }
        return null;
      }
    }
    return null;
  }

  public boolean isCellEditable(int row, int col) {
    return false;
  }

  Round getRound() {
    return round;
  }

  void setClub(Club club) {
    this.club = club;
    update();
  }

  /*
   * Removes all players that are not in the selected club.
   * If club == null all players are shown.
   */
  private void filter(Players players) {
    if (club != null) {
      for (int i=players.size()-1; i>=0; i--) {
        Player player = players.get(i);
        if (player.getClub() != club) {
          players.remove(player);
        }
      }
    }
  }
}
