package cp.pairing;

import cp.ChessPairerException;
import cp.Game;
import cp.Games;
import cp.Player;
import cp.Players;
import cp.ScoreLotnrComparator;
import cp.Tournament;

import java.util.Collections;
import java.util.Vector;

public class Berger extends PairingSystem {
  
  public Berger(Tournament tournament) {
    super(tournament);
    initialize();
  }
  
  public Games pair() throws ChessPairerException {
    // First sort player according to tournament setting
    Players players = tournament.getPlayers();
    Collections.sort(players, new ScoreLotnrComparator(null));
    
    if (players.size() % 2 == 1) {
      players.add(null);
    }
    Games games = pair(tournament.getPlayers());
    if (players.lastElement() == null) {
      players.remove(null);
    }
    return games;
  }

  protected Games pair(Players players) throws ChessPairerException {
    int gamesPerRound = (int) Math.ceil(players.size()/2.0);
    // Make upper & lower row of players
    Vector upper = new Vector();
    Vector lower = new Vector();
    for (int i=0; i<gamesPerRound; i++) {
      int pos;
      if (i % 2 == 0) {
        pos = i/2;
      }
      else {
        pos = players.size()/2 + i/2;        
      }
      upper.add(players.get(pos));
      lower.add(players.get(players.size()-1-pos));
    }
    // Loop until correct round
    int roundNr = tournament.getRounds().size()+1;
    for (int i=1; i<roundNr; i++) {
      rotate(upper, lower);
    }
    return pair(upper, lower, roundNr);
  }

  private void rotate(Vector upper, Vector lower) {
    Object player = upper.firstElement();
    lower.insertElementAt(player, 1);
    upper.remove(player);
    player = lower.lastElement();
    upper.addElement(player);
    lower.remove(player);
  }

  private Games pair(Vector upper, Vector lower, int roundNr) throws ChessPairerException {
    Games games = new Games(tournament);
    for (int i=0; i<upper.size(); i++) {
      // White player = odd positions in upper except first game. Then player has white at odd rounds.
      Game game;
      if (roundNr % 2 == 1) {
        // Odd round. Players 0,2,4... has white.
        if (i % 2 == 0) {
          game = new Game(games, (Player) upper.get(i), (Player) lower.get(i));
        }
        else {
          game = new Game(games, (Player) lower.get(i), (Player) upper.get(i));
        }
      }
      else {
        // Even round. Players 2,4,6... has white.
        if (i % 2 == 0 && i>=2) {
          game = new Game(games, (Player) upper.get(i), (Player) lower.get(i));
        }
        else {
          game = new Game(games, (Player) lower.get(i), (Player) upper.get(i));
        }
      }
      games.add(game);
    }    
    return games;
  }
}
