package cp.tiebreak;

import cp.Game;
import cp.Player;
import cp.Round;

import java.util.Vector;

public class TiebreakBuchholz implements Tiebreak {
  
  /**
   * Calculates the tiebreak score for a player up to a given round.
   * 
   */
  public double getTiebreak(Round round, Player player) {
    double tiebreak = 0;
    Vector games = player.getGames();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);     
      if (game.getRoundNr() <= round.getRoundNr()) {
        if (!game.plays(null)) {
          Player white = game.getWhitePlayer();
          Player black = game.getBlackPlayer();
          if (white == player) {
            tiebreak += getTiebreakScore(round, black);
          }
          else if (black == player) {
            tiebreak += getTiebreakScore(round, white);
          }
        }
        else {
          tiebreak += 0.5*round.getRoundNr();  // If meeting bye or wo its worth rounds/2
        }
      }
    }
    return tiebreak;
  }
  
  /**
   * Calculates the tiebreak score for an opponent player.
   * Not played games is calculated as  point, not 1 or 0 points.
   * Therefore this score may be different from the players score.
   */
  protected double getTiebreakScore(Round round, Player player) {
    double ans = 0;
    Vector games = player.getGames();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (game.getRoundNr() <= round.getRoundNr()) {
        if (game.plays(null) || game.wonOnWO(player) || game.getResult().isToBePlayed()) {
          ans += 0.5; // If it's a bye or not played yet its worth 0.5 tiebreak point
        }
        else {
          ans += game.getPairingScore(player, round.getRoundNr());
        }
      }
    }
    return ans;
  }
  
  public String toString() {
    return "Buchholz";
  }

  public String getShortName() {
    return "Buch";
  }

  public String format(double value) {
    return Double.toString(value);
  }

}


