package cp.tiebreak;

import cp.Game;
import cp.Games;
import cp.Player;
import cp.Round;
import cp.Rounds;

import java.util.Vector;

public class TiebreakInternalMeeting implements Tiebreak {
  
  /**
   * Calculates the tiebreak score for a player up to a given round.
   */
  public double getTiebreak(Round round, Player player) {
		double score = player.getScore(round);
    Vector<Player> players = round.getTournament().getPlayers();
    // First get all players that have the same score.
    Vector<Player> sameScore = new Vector<Player>();
    for (int i=0; i<players.size(); i++) {
      Player pl = players.get(i);
      if (pl != player && pl.getScore(round) == score) {
        sameScore.add(pl);
      }
    }
    // For each tiebreak before internal meeting, remove players that does not have the same tiebreak.
    Vector<Tiebreak> tiebreaks = round.getTournament().getSelectedTiebreaks();
    for (int i=0; i<tiebreaks.size(); i++) {
      Tiebreak tb = tiebreaks.get(i);
      if (tb == this) {
        break;
      }
      double aTiebreak = player.getTiebreak(tb, round);
      for (int j=sameScore.size()-1; j>=0; j--) {
        Player pl = sameScore.get(j);
        double bTiebreak = pl.getTiebreak(tb, round);
        if (aTiebreak != bTiebreak) {
          sameScore.remove(pl);
        }
      }
    }
    // Now we have the same score and the same tiebreak up until this tiebreak. Calculate tiebreak.
    double tiebreak = 0;
		// For each opponent that has the same score, add the internal result.
    Vector games = player.getGames();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (game.getRoundNr() <= round.getRoundNr()) {
				if (!game.plays(null)) {
					Player white = game.getWhitePlayer();
					Player black = game.getBlackPlayer();
					if (white == player && sameScore.contains(black)) {
						tiebreak += game.getResult().getWhiteScore();
					}
					else if (black == player && sameScore.contains(white)) {
				    tiebreak += game.getResult().getBlackScore();
					}
				}
      }
    }
    return tiebreak;
  }
  
  public String toString() {
    return "Inbrdes mte";
  }

  public String getShortName() {
    return "Inb";
  }
  
  public String format(double value) {
    return Double.toString(value);
  }

}
