package cp.tiebreak;

import cp.Game;
import cp.Player;
import cp.Round;

import java.util.Vector;

public class TiebreakSSFBuchholz extends TiebreakBuchholz implements Tiebreak {
  /**
   * Calculates the tiebreak score for a player up to a given round.
   * SSF version removes the lowest opponent score and gives no score for byes.
   */
  public double getTiebreak(Round round, Player player) {
    double min = Double.POSITIVE_INFINITY;
    double tiebreak = 0;
    Vector games = player.getGames();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);     
      if (game.getRoundNr() <= round.getRoundNr()) {
        double tiebreakScore = 0;
        if (!game.plays(null)) {
          Player white = game.getWhitePlayer();
          Player black = game.getBlackPlayer();
          if (white == player) {
            tiebreakScore = getTiebreakScore(round, black);
            tiebreak += tiebreakScore;
          }
          else if (black == player) {
            tiebreakScore = getTiebreakScore(round, white);
            tiebreak += tiebreakScore;
          }
        }
        if (tiebreakScore < min) {
          min = tiebreakScore;
        }
      }
    }
    if (round.getRoundNr() > 1) {
      return Math.max(tiebreak - min, 0);
    }
    return tiebreak;
  }
  
  /**
   * Calculates the tiebreak score for an opponent player.
   * Not played games is calculated as  point, not 1 or 0 points.
   * Bye is calculated as 1 point, not  point as in Buchholz.
   * Therefore this score may be different from the players score.
   */
  protected double getTiebreakScore(Round round, Player player) {
    double ans = 0;
    Vector games = player.getGames();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (game.getRoundNr() <= round.getRoundNr()) {
        if (game.plays(null)) {
          ans += 1; 
        }
        else if (game.getResult().isToBePlayed()) {
          ans += 0.5; // If it's not played its worth 0.5 tiebreak point
        }
        else {
          ans += game.getPairingScore(player, round.getRoundNr());
        }
      }
    }
    return ans;
  }
  
  public String toString() {
    return "SSF Buchholz";
  }

  public String getShortName() {
    return "SSFBuch";
  }
}
